package Sophos::Reports::Report::CategoriesByGroup;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use PureMessage::Version qw($PREFIX);
use Sophos::Reports::DataManip qw(rotate_pivot_on_1 rotate);
use Sophos::Reports::MsgTypes qw(msg_clrs msg_names);
use Carp qw(croak);

sub new {
    my ($class,%opts) = @_;

    my $self = $class->SUPER::new(%opts);

    croak "user object required" unless $self->{user};

    my @fields = qw(blocked spam virus delivered other);
    $self->{fields} = \@fields;

    $self->{lines}{y_label} = 'Number of Messages'; # XXX localize
    $self->{graph}{dclrs} = msg_clrs(@fields);
    $self->{legend} = msg_names(@fields);
    $self->{table_row_subtotals} = 1;
    $self->{column_title} = 'Type'; # XXX localize
    $self->{hide_groups} = 1;
    $self->{views} = {
       bars  => 1,
       table => 1,
    };

    $self->{type} ||= 'bars';

    return $self;
}

sub groups {
    my $self = shift;
    my @groups = grep { $_ ne 'supergroup' } 
                   map { $_->group_name }
                     $self->{user}->groups;
    return @groups if wantarray;
    return join ',', @groups;
}

sub data {
    my ($self, %opts) = @_;
    
    delete $self->{only_group};

    my $type = $opts{type} || $self->{type};

    my ($pa,$pb) = $self->time_period;
    my $cases    = $self->type_cases('type');

    my $dbh = $self->dbh;

    my $direction_sql = '';
    if ($self->{direction}) {
        my $inbound = $self->{direction} eq 'inbound' ? 'true' : 'false';
        $direction_sql = "AND inbound = '$inbound'";
    }

    my $groups = join ',', map { "'$_'" } $self->groups;

    my $sql = <<__SQL__;
SELECT group_name,
$cases
FROM (
    SELECT group_name,
           type,
           SUM(tally) AS tally
    FROM   prd_msgs_by_type
    WHERE  period_start >= $pa
      AND period_start < $pb
      AND group_name IN ($groups)
      $direction_sql
    GROUP BY group_name, type
) AS X
GROUP BY group_name
ORDER BY group_name
__SQL__

    return $sql if $opts{get_sql};

    my $results = $groups ? $dbh->selectall_arrayref($sql) : [()];
    return $results if $opts{no_flop};

    my $glist = [];
    my $i = -1; 
    my @data;
    for my $row (@{$results}) {
        $glist->[++$i] = shift @$row;

        for (0 .. $#$row) {
            $data[$_][$i] = $row->[$_] if defined $row->[$_];
        }
    }

    $self->clean_data(\@data,
        rows => scalar @{$self->{fields}},
        cols => scalar @$glist,
        type => $type
    ) or return undef;

    return [$glist,@data];
}

1;
