package Sophos::Reports::Report::CustomMarkHits;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use PureMessage::Reports::Custom qw(get_report_data);
use PureMessage::Version qw($PREFIX);

sub new {
    my ($class, %opts) = @_;
    my $self = $class->SUPER::new(%opts);

    $self->{type} ||= 'table';
    $self->{table_row_subtotals} = 1;

    die("Must specify a mark key") unless defined $opts{mkey};

    $self->{bars}{x_labels_vertical} = 1;
    delete $self->{bars}{bar_spacing};
    $self->{_mkey}           = $opts{mkey};
    my $rep_cfg              = get_report_data($opts{mkey});
    die("Unable to get report data")
        unless defined $rep_cfg->{'values'};
    die("Report must contain at least one value")
        unless (exists $rep_cfg->{'values'} && @{$rep_cfg->{'values'}});


    $self->{fields}          = $rep_cfg->{'values'};
    $self->{legend}          = $self->{fields};
    $self->{column_title}    = 'Time'; # For CSV
    $self->{report_heading}  = $rep_cfg->{'title'};

    $self->{views} = {
       table => 1,
       bars  => 1,
    };

    $self->{title} = "/groups/images/$self->{lang}/ttl_mt_customPolicyReport.png";
    $self->{lines}{y_label} = 'Number of Messages';

    return $self;
}

sub has_direction { 0 }

sub data {
    my ($self, %opts) = @_;

    my $csv  = $opts{csv};
    my $type = $opts{type} || $self->{type};

    my $dbh = $self->dbh;

    my ($pa,$pb)         = $self->time_period;
    my $group_sql        = $self->_group_sql();
    my $cases            = $self->type_cases('mark_val');
    my ($interval, $fmt) = $self->granularity($csv ? (full_date => 1) : ());
    my $first_field      = $self->{fields}->[0];

    my $sql = <<__SQL__;
SELECT to_char(date_trunc('$interval', period_start), '$fmt'),
$cases
FROM (
    SELECT date_trunc('$interval', period_start) AS period_start, mark_val,
           SUM(tally) AS tally
    FROM   prd_custom_mark_hits
    WHERE  period_start >= $pa
      AND period_start < $pb
      AND mark_key = $self->{_mkey}
      $group_sql
    GROUP BY date_trunc('$interval', period_start), mark_val
UNION
    SELECT date_trunc('$interval',  ts::timestamp) AS period_start, '$first_field' AS mark_val, 0
    FROM   generate_series($pa, $pb, '1 $interval') AS ts
    WHERE  ts >= $pa
    AND    ts < $pb
    GROUP BY date_trunc('$interval', ts::timestamp)
) AS X
GROUP BY date_trunc('$interval', period_start)
ORDER BY date_trunc('$interval', period_start);
__SQL__

    return $sql if $opts{get_sql};

    my $result = $dbh->selectall_arrayref($sql);
    return $result if $opts{no_flop};

    my $time = [];
    my $i = -1;
    my @data;
    for my $row (@{$result}) {
        $time->[++$i] = shift @$row;

        for (0 .. $#$row) {
            $data[$_][$i] = $row->[$_] if defined $row->[$_];
        }
    }

    $self->clean_data(\@data,
        rows => scalar @{$self->{fields}},
        cols => scalar @$time,
        type => $type
    ) or return undef;

    return [$time,@data];
}

1;
