package Sophos::Reports::Report::MessageCategories;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use PureMessage::Version qw($PREFIX);
use Sophos::Reports::DataManip qw(rotate_pivot_on_1 rotate);
use Sophos::Reports::MsgTypes qw(msg_clrs msg_names);

sub new {
    my ($class,%opts) = @_;
    my $self = $class->SUPER::new(%opts);

    $self->{lines}{y_label} = 'Number of Messages'; # XXX localize

    my @fields = qw(blocked spam virus delivered other);
    $self->{fields} = \@fields;

    $self->{bars}{x_labels_vertical} = 1;
    delete $self->{bars}{bar_spacing};

    $self->{graph}{dclrs} = msg_clrs(@fields);
    $self->{legend} = msg_names(@fields);
    $self->{table_row_subtotals} = 1;
    $self->{column_title} = 'Time'; # XXX localize
    $self->{views} = {
       bars => 1,
       table => 1,
    };

    $self->{type} ||= 'bars';

    # super will setup lang
    $self->{title} = "/groups/images/$self->{lang}/ttl_mt_msgCategory.png";

    return $self;
}

sub data {
    my ($self, %opts) = @_;

    my $csv  = $opts{csv};
    my $type = $opts{type} || $self->{type};

    my $dbh = $self->dbh;

    my $direction_sql = '';
    if ($self->{direction}) {
        my $inbound = $self->{direction} eq 'inbound' ? 'true' : 'false';
        $direction_sql = "AND inbound = '$inbound'";
    }

    my ($pa,$pb)  = $self->time_period;
    my $group_sql = $self->_group_sql();
    my $cases     = $self->type_cases('type');

    my ($interval, $fmt) = $self->granularity($csv ? (full_date => 1) : ());
    my $first_field = $self->{fields}->[0];

    my $sql = <<__SQL__;
SELECT to_char(date_trunc('$interval', period_start), '$fmt'),
$cases
FROM (
    SELECT date_trunc('$interval', period_start) AS period_start, type,
           SUM(tally) AS tally
    FROM   prd_msgs_by_type
    WHERE  period_start >= $pa
      AND period_start < $pb
      $direction_sql
      $group_sql
    GROUP BY date_trunc('$interval', period_start), type
UNION
    SELECT date_trunc('$interval',  ts::timestamp) AS period_start, '$first_field' AS type, 0
    FROM   generate_series($pa, $pb, '1 $interval') AS ts
    WHERE  ts >= $pa
    AND    ts < $pb
    GROUP BY date_trunc('$interval', ts::timestamp)
) AS X
GROUP BY date_trunc('$interval', period_start)
ORDER BY date_trunc('$interval', period_start)
__SQL__

    return $sql if $opts{get_sql};

    return $dbh->selectall_arrayref($sql) if $opts{no_flop};

    my $time = [];
    my $i = -1; 
    my @data;
    for my $row (@{$dbh->selectall_arrayref($sql)}) {
        $time->[++$i] = shift @$row;

        for (0 .. $#$row) {
            $data[$_][$i] = $row->[$_] if defined $row->[$_];
        }
    }

    $self->clean_data(\@data, rows => scalar @{$self->{fields}},
                              cols => scalar @$time,
                              type => $type
                     ) or return undef;

    return [$time,@data];
}

1;
