package Sophos::Reports::Report::MtaRejected;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use Sophos::Reports::MsgTypes qw(msg_clrs msg_names);

sub new {
    my ($class,%opts) = @_;
    my $self = $class->SUPER::new(%opts);

    my @fields = qw(rejected allowed);
    $self->{fields} = \@fields;

    $self->{lines}{y_label} = 'Number of Connections'; # XXX localize
    $self->{graph}{dclrs} = msg_clrs(@fields);
    $self->{legend} = msg_names(@fields);
    $self->{table_row_subtotals} = 1;
    $self->{column_title} = 'Time'; # XXX localize
    $self->{hide_groups} = 1;

    $self->{bars}{x_labels_vertical} = 1;
    delete $self->{bars}{bar_spacing};

    $self->{views} = {
        bars => 1,
        table => 1,
    };

    $self->{type} ||= 'bars';

    return $self;
}

sub has_direction { 0 }

sub data {
    my ($self, %opts) = @_;
    
    my $csv  = $opts{csv};
    my $type = $opts{type} || $self->{type};

    my ($pa,$pb) = $self->time_period;

    my $dbh = $self->dbh;
    my ($interval, $fmt) = $self->granularity($csv ? (full_date => 1) : ());
    
    my $sql = <<__SQL__;
SELECT to_char(date_trunc('$interval', period_start), '$fmt'),
    COALESCE(SUM(CASE WHEN action IN ('reject', 'tempfail') THEN tally END),0) AS rejected,
    COALESCE(SUM(CASE action WHEN 'ok' THEN tally END),0) AS allowed 
FROM (
    SELECT date_trunc('$interval', period_start) AS period_start, action,
           SUM(tally) AS tally
    FROM   prd_connections_by_action
    WHERE  period_start >= $pa
      AND period_start < $pb
    GROUP BY date_trunc('$interval', period_start), action
UNION
    SELECT date_trunc('$interval', ts::timestamp) AS period_start, 'ok' AS action, 0
    FROM   generate_series($pa, $pb, '1 $interval') AS ts
    WHERE  ts >= $pa
    AND    ts < $pb
    GROUP BY date_trunc('$interval', ts::timestamp)
) AS X
GROUP BY date_trunc('$interval', period_start)
ORDER BY date_trunc('$interval', period_start)
__SQL__

    return $sql if $opts{get_sql};

    return $dbh->selectall_arrayref($sql) if $opts{no_flop};
    
    my $time = [];
    my $i=-1;
    my @data;
    for my $row (@{$dbh->selectall_arrayref($sql)}) {
        $time->[++$i] = shift @$row;

        for (0 .. $#$row) {
            $data[$_][$i] = $row->[$_] if defined $row->[$_];
        }
    }

    $self->clean_data(\@data, rows => 2,
                              cols => scalar @$time,
                              type => $type
                     ) or return undef;

    return [$time,@data];
}

1;

