package Sophos::Reports::Report::PolicyMarkHits;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use PureMessage::Version qw($PREFIX);

sub new {
    my ($class, %opts) = @_;
    my $self = $class->SUPER::new(%opts);

    $self->{bars}{x_labels_vertical} = 1;
    delete $self->{bars}{bar_spacing};
    $self->{column_title} = 'Mark Hits';

    $self->{type} ||= 'table';
    $self->{views} = {
		bars => 1,
		table => 1,
    };

    return $self;
}

sub _type_sql {
    my $self = shift;
    return "";
}

sub has_direction { 0 }

sub data {
    my ($self, %opts) = @_;
    my $type = $opts{type} || $self->{type};
    my ($pa,$pb) = $self->time_period;
    my $dbh = $self->dbh;

    my $group_sql = $self->_group_sql();

    my $type_sql = $self->_type_sql();

    my $sql = <<__SQL__;
SELECT policy_mark, SUM(tally) AS tally
FROM   prd_policy_mark_hits
WHERE  period_start >= $pa
  AND period_start < $pb
      $type_sql
      $group_sql
GROUP BY policy_mark
ORDER BY tally DESC, policy_mark
__SQL__

    return $sql if $opts{get_sql};

    my $results = $dbh->selectall_arrayref($sql);
    return $results if $opts{no_flop} or $opts{csv};

    my $marks = [];
    my $i = -1;
    my @data;

    for my $row (@{$results}) {
        $marks->[++$i] = shift @$row;
        for (0 .. $#$row) {
        $data[$_][$i] = $row->[$_] if defined $row->[$_];
        }
    }

    $self->clean_data(\@data,
        rows => 1,
        cols => scalar @$marks,
        type => $type
    ) or return undef;

    return [$marks, @data];
}

1;
