package Sophos::Reports::Report::TopRelays;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use PureMessage::Version qw($PREFIX);

sub new {
    my ($class, %opts) = @_;
    my $self = $class->SUPER::new(%opts);

    $self->{type} ||= 'table';
    $self->{views} = {
       table => 1,
    };

    return $self;
}

sub _type_sql {
    my $self = shift;
    return "";
}

sub data {
    my ($self, %opts) = @_;

    my $type = $opts{type} || $self->{type};

    my ($pa,$pb) = $self->time_period;

    my $dbh = $self->dbh;

    my $direction_sql = '';
    if ($self->{direction}) {
        my $inbound = $self->{direction} eq 'inbound' ? 'true' : 'false';
        $direction_sql = "AND inbound = '$inbound'";
    }

    my $group_sql = $self->_group_sql();
    my $type_sql  = $self->_type_sql();

    my $sql = <<__SQL__;
SELECT relay, SUM(tally) AS tally
FROM   prd_relays_by_type
WHERE  period_start >= $pa
  AND period_start < $pb
      $type_sql
      $direction_sql
      $group_sql
GROUP BY relay
ORDER BY tally DESC, relay
LIMIT 20
__SQL__

    return $sql if $opts{get_sql};

    my $data = $dbh->selectall_arrayref($sql);
    return scalar(@$data) ? $data : undef;
}

1;
