package Sophos::Reports::Report::TopVirusTypes;
use strict;
use warnings;
use base 'Sophos::Reports::Report::PMX';

use PureMessage::Version qw($PREFIX);

sub new {
    my ($class, %opts) = @_;
    my $self = $class->SUPER::new(%opts);

    $self->{bars}{x_labels_vertical} = 1;
    delete $self->{bars}{bar_spacing};
    $self->{column_title} = 'Virus Type'; # For CSV

    $self->{type} ||= 'table';
    $self->{views} = {
       bars  => 1,
       table => 1,
    };

    # super will setup lang
    $self->{title} = "/groups/images/$self->{lang}/ttl_mt_topVirusType.png",

    return $self;
}

sub data {
    my ($self, %opts) = @_;

    my $type = $opts{type} || $self->{type};

    my ($pa,$pb) = $self->time_period;

    my $dbh = $self->dbh;

    my $direction_sql = '';
    if ($self->{direction}) {
        my $inbound = $self->{direction} eq 'inbound' ? 'true' : 'false';
        $direction_sql = "AND inbound = '$inbound'";
    }

    my $group_sql = $self->_group_sql();

    my $sql = <<__SQL__;
SELECT virus_name, SUM(tally) AS tally
FROM   prd_msgs_by_virus
WHERE  period_start >= $pa
  AND period_start < $pb
      $direction_sql
      $group_sql
GROUP BY virus_name
ORDER BY tally DESC, virus_name
__SQL__

    return $sql if $opts{get_sql};

    my $result = $dbh->selectall_arrayref($sql);
    return $result if $opts{no_flop} or $opts{csv};

    my $viruses = [];
    my $i = -1;
    my @data;
    for my $row (@{$result}) {
        $viruses->[++$i] = shift @$row;

        for (0 .. $#$row) {
            $data[$_][$i] = $row->[$_] if defined $row->[$_];
        }
    }

    $self->clean_data(\@data,
        rows => 1,
        cols => scalar @$viruses,
        type => $type
    ) or return undef;

    return [$viruses, @data];
}

1;
